#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
A bible processing toolkit with free bible data for Python
Latest version can be found at https://github.com/neocl/freebible

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2018, Le Tuan Anh <tuananh.ke@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

########################################################################

import os
import logging

from freebible.model import Collection, Book, Chapter, Verse
from chirptext.io import read_file


# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

def getLogger():
    return logging.getLogger(__name__)


STANDARD_BOOK_NAMES = {'Ge': 'Gen', 'Exo': 'Exo', 'Lev': 'Lev', 'Num': 'Num', 'Deu': 'Deut',
                       'Josh': 'Josh', 'Jdgs': 'Judg', 'Ruth': 'Rth', '1Sm': '1 Sam',
                       '2Sm': '2 Sam', '1Ki': '1 Kgs', '2Ki': '2 Kgs', '1Chr': '1 Chron',
                       '2Chr': '2 Chron', 'Ezra': 'Ezra', 'Neh': 'Neh', 'Est': 'Esth',
                       'Job': 'Job', 'Psa': 'Pslm', 'Prv': 'Prov', 'Eccl': 'Eccles',
                       'SSol': 'Song', 'Isa': 'Isa', 'Jer': 'Jer', 'Lam': 'Lam', 'Eze': 'Ezek',
                       'Dan': 'Dan', 'Hos': 'Hos', 'Joel': 'Joel', 'Amos': 'Amos', 'Obad': 'Obad',
                       'Jonah': 'Jnh', 'Mic': 'Micah', 'Nahum': 'Nah', 'Hab': 'Hab', 'Zep': 'Zeph',
                       'Hag': 'Haggai', 'Zec': 'Zech', 'Mal': 'Mal', 'Mat': 'Matt', 'Mark': 'Mrk',
                       'Luke': 'Luk', 'John': 'John', 'Acts': 'Acts', 'Rom': 'Rom', '1Cor': '1 Cor',
                       '2Cor': '2 Cor', 'Gal': 'Gal', 'Eph': 'Ephes', 'Phi': 'Phil', 'Col': 'Col',
                       '1Th': '1 Thess', '2Th': '2 Thess', '1Tim': '1 Tim', '2Tim': '2 Tim',
                       'Titus': 'Titus', 'Phmn': 'Philem', 'Heb': 'Hebrews', 'Jas': 'James',
                       '1Pet': '1 Pet', '2Pet': '2 Pet', '1Jn': '1 John', '2Jn': '2 John',
                       '3Jn': '3 John', 'Jude': 'Jude', 'Rev': 'Rev'}


# -------------------------------------------------------------------------------
# Functions
# -------------------------------------------------------------------------------

def parse_kougo(seisho_path):
    ''' Parse Kougo from raw text format '''
    if not seisho_path or not os.path.isfile(seisho_path):
        raise Exception("Seisho not found (provided path: {})".format(seisho_path))
    return parse_kougo_raw(read_file(seisho_path))


def parse_kougo_raw(kougo_content):
    lines = kougo_content.splitlines()
    getLogger().debug("Found {} verses.".format(len(lines)))
    assert lines[0].strip().endswith('=000 Bible begins')
    current_book = None
    books = []
    for idx, l in enumerate(lines[1:]):
        if l.startswith("="):
            if l == '=ALL':
                continue
            book = Book.from_string(l)
            if book is not None:
                books.append(book)
                current_book = book
        elif l == 'END ':
            if current_book is not None:
                current_book = None
            pass
        else:
            # a verse line
            v = Verse.from_string(l)
            if v is not None:
                if current_book is None:
                    getLogger().warning("Found an orphant verse: {}".format(v))
                elif v.chapID not in current_book:
                    current_book.add_chapter(Chapter(ID=v.chapID))
                # verify short name
                if not current_book.short_name:
                    current_book.short_name = v.book_key
                elif current_book.short_name != v.book_key:
                    getLogger().warning("Inconsistent book_key: {} vs {}".format(current_book.short_name, v.book_key))
                current_book[v.chapID].add_verse(v)
            else:
                print("ERROR:", idx, repr(l))
    # books to collection
    seisho = Collection()
    for b in books:
        seisho.add_book(b)
        # also add standard book's short name
        std_name = STANDARD_BOOK_NAMES[b.short_name]
        if std_name != b.short_name:
            seisho.book_map[std_name] = seisho[b.short_name]
    return seisho



# -*- coding: utf-8 -*-

'''
Bible models
Latest version can be found at https://github.com/neocl/freebible

:copyright: (c) 2018, Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

import os
import logging
import re
import json
try:
    import yaml
except:
    logging.getLogger(__name__).warning("YAML module is not available. Exporting to YAML function will NOT work.")

from chirptext.anhxa import to_obj

# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

DATA_FOLDER = os.path.abspath(os.path.expanduser('./data'))


def getLogger():
    return logging.getLogger(__name__)


# -------------------------------------------------------------------------------
# Data structures
# -------------------------------------------------------------------------------

class Collection(object):
    ''' Book collection '''
    def __init__(self, name=None, books=None):
        self.book_map = {}
        self.bookid_map = {}
        self.books = []
        self.name = name
        if books is not None:
            for b in books:
                self.add_book(b)

    def __len__(self):
        return len(self.books)

    def __getitem__(self, key):
        return self.book_map[key]

    def __contains__(self, item):
        return item in self.book_map

    def __iter__(self):
        return iter(self.books)

    def quote(self, book_key, cid=None, vid=None):
        if book_key and cid and vid:
            return self[book_key][cid][vid]
        elif book_key and cid:
            return self[book_key][cid]
        elif book_key:
            return self[book_key]
        else:
            raise KeyError("Book key cannot be None")

    def add_book(self, book):
        self.book_map[book.short_name] = book
        self.bookid_map[book.ID] = book
        self.books.append(book)
        return book

    def keys(self):
        ''' Get a list of all available book keys (short names ) '''
        return [b.short_name for b in self]

    def summarise(self):
        cc = sum(len(b) for b in self)
        cv = sum(b.verse_count() for b in self.books)
        print("Available books: {}".format(len(self.books)))
        print("Available chaps: {}".format(cc))
        print("Available verses: {}".format(cv))

    @staticmethod
    def read_json(self, json_data):
        ''' Read bible as a Book Collection from json data '''
        books = [Book.from_json(b) for b in json_data]
        bible = Collection(books)
        return bible

    @staticmethod
    def read_json_file(self, json_filepath):
        ''' Read bible from a JSON file '''
        with open(json_filepath, 'r') as infile:
            return Collection.read_json(json.loads(infile.read()))

    def export(self, output, format='json'):
        ''' Export bible data to diffrent formats '''
        json_books = [b.to_json() for b in self]
        if format == 'json':
            output.write(json.dumps(json_books, ensure_ascii=False, indent=2))
        elif format == 'yaml':
            try:
                yaml_str = yaml.safe_dump(json_books, default_flow_style=False, allow_unicode=True, encoding='utf-8')
                output.write(yaml_str)
            except:
                getLogger().exception("Error was raised while exporting to YAML format")
        elif format == 'txt':
            for b in self:
                for c in b:
                    for v in c:
                        output.write(v.text)
                        output.write('\n')
        else:
            raise Exception("Format {} is not supported".format(format))


class Book(object):

    TITLE_PTN = re.compile('=(?P<id>\d+) (?P<bookname>\w+) (?P<title>\w+(?: \(\w+\))?) - (?P<title_eng>[\w ]+)')

    def __init__(self, ID='', title='', filename='', title_eng='', short_name=''):
        """
        """
        self.ID = str(int(ID))
        self.filename = filename
        self.title = title
        self.title_eng = title_eng
        self.short_name = short_name
        self.__chapters = []
        self.__chapter_map = {}

    def add_chapter(self, chapter):
        if chapter.ID in self.__chapter_map:
            raise Exception("Chapter {} exists.".format(chapter.ID))
        else:
            self.__chapters.append(chapter)
            self.__chapter_map[chapter.ID] = chapter
            chapter.book = self
            return chapter

    def __getitem__(self, chapID):
        if chapID not in self.__chapter_map:
            return self.__chapter_map[str(chapID)]
        else:
            return self.__chapter_map[chapID]

    def __contains__(self, chapID):
        return chapID in self.__chapter_map

    def verse_count(self):
        return sum(len(c) for c in self)

    def __len__(self):
        return len(self.__chapters)

    def chapters(self):
        return list(self.__chapters)

    def __iter__(self):
        return iter(self.__chapters)

    @staticmethod
    def from_string(title_string):
        m = Book.TITLE_PTN.match(title_string)
        if m:
            return Book(ID=m.group('id'),
                        title=m.group('title'),
                        filename=m.group('bookname'),
                        title_eng=m.group('title_eng'))
        else:
            return None

    def __repr__(self):
        return "Book(ID={}, title={}, filename={}, title_eng={}, short_name={})".format(
            repr(self.ID), repr(self.title), repr(self.filename), repr(self.title_eng), repr(self.short_name))

    def __str__(self):
        return "[{}] {}".format(self.ID, self.title)

    def to_json(self):
        return {'ID': self.ID, 'filename': self.filename, 'title': self.title, 'title_eng': self.title_eng, 'short_name': self.short_name, 'chapters': [c.to_json() for c in self]}

    @staticmethod
    def from_json(json_dict):
        book = to_obj(Book, json_dict, 'ID', 'filename', 'title', 'title_eng', 'short_name')
        for chap in json_dict['chapters']:
            book.add_chapter(Chapter.from_json(chap))
        return book


class Chapter(object):

    def __init__(self, ID=None, title='', book=None):
        """
        """
        self.ID = ID
        self.title = title
        self.__verses = []
        self.__verse_map = {}
        self.__book = book

    @property
    def book(self):
        return self.__book

    @book.setter
    def book(self, value):
        self.__book = value

    def add_verse(self, verse):
        self.__verse_map[verse.ID] = verse
        self.__verses.append(verse)

    def __getitem__(self, key):
        if key not in self.__verse_map:
            return self.__verse_map[str(key)]
        else:
            return self.__verse_map[key]

    def __contains__(self, item):
        return item in self.__verse_map

    def verses(self):
        return list(self.__verses)

    def __len__(self):
        return len(self.__verses)

    def __iter__(self):
        return iter(self.__verses)

    def __repr__(self):
        return "Chapter(ID={})".format(repr(self.ID))

    def __str__(self):
        if self.__book:
            return "{} - chapter {}".format(self.book.title, self.ID)
        else:
            return "chapter {}".format(self.ID)

    def to_json(self):
        return {'ID': self.ID,
                'title': self.title,
                'verses': [v.to_json() for v in self]}

    @staticmethod
    def from_json(json_dict):
        chapter = to_obj(Chapter, json_dict, 'ID', 'title')
        # add verses
        for v in json_dict['verses']:
            chapter.add_verse(Verse.from_json(v))
        return chapter


class Verse(object):
    def __init__(self, ID='', text='', chapID='', book_key=''):
        """
        """
        self.ID = ID
        self.text = text
        self.chapID = chapID
        self.book_key = book_key

    def __str__(self):
        return "[{} {}:{}] {}".format(self.book_key, self.chapID, self.ID, self.text)

    def to_json(self):
        return {'ID': self.ID, 'text': self.text}

    def __repr__(self):
        return "{} {}:{} {}".format(self.book_key, self.chapID, self.ID, self.text)

    @staticmethod
    def from_json(json_dict):
        return to_obj(Verse, json_dict, 'ID', 'text', 'chapID', 'book_key')

    VERSE_PTN = re.compile('(?P<book_key>\w+) (?P<cid>\d+):(?P<vid>\d+) (?P<bookname>\w+) (?P<cid2>\d+):(?P<vid2>\d+) (?P<verse_text>.*)\s*')

    @staticmethod
    def from_string(verse_string):
        m = Verse.VERSE_PTN.match(verse_string)
        return Verse(ID=m.group('vid'), text=m.group('verse_text'), chapID=m.group('cid'), book_key=m.group('book_key'))


class BookMap(object):
    ''' Map book short names to standard book ID (WEB) '''
    def __init__(self, abbr_data):
        self.abbr_data = list(abbr_data)
        self.name_to_bid_map = {r['text']: r['bookID'] for r in self.abbr_data}
        self.bid_to_std_name_map = {r['bookID']: r['text'] for r in self.abbr_data if r['is_standard'] == '1'}
        # these abbrs are missing from WEB keys
        self.name_to_std_name_map = {'Deu': 'Deut',
                                     'Ruth': 'Rth',
                                     '1Sm': '1 Sam',
                                     '2Sm': '2 Sam',
                                     'Est': 'Esth',
                                     'Eccl': 'Eccles',
                                     'SSol': 'Song',
                                     'Jonah': 'Jnh',
                                     'Nahum': 'Nah',
                                     'Mat': 'Matt',
                                     'Mark': 'Mrk',
                                     'Luke': 'Luk',
                                     'Phi': 'Phil',
                                     'Phmn': 'Philem'}
        for row in abbr_data:
            self.name_to_std_name_map[row['text']] = self.bid2name(row['bookID'])

    def bid2name(self, bookID):
        ''' get standard short name for a bookID '''
        if bookID in self.bid_to_std_name_map:
            return self.bid_to_std_name_map[bookID]
        return None

    def name2bid(self, short_name):
        ''' Short name to bookID '''
        if short_name in self.name_to_bid_map:
            return self.name_to_bid_map[short_name]
        return None

    def standardize(self, short_name):
        if short_name in self.name_to_std_name_map:
            return self.name_to_std_name_map[short_name]
        else:
            return None



import os

MY_DIR = os.path.dirname(__file__)
KOUGO_PATH = os.path.join(MY_DIR, 'kougo', 'kougo.txt.gz')
WEB_VERSES = os.path.join(MY_DIR, 'web', 't_web.csv.gz')
WEB_BOOKS = os.path.join(MY_DIR, 'web', 'key_english.csv.gz')
WEB_ABBRS = os.path.join(MY_DIR, 'web', 'key_abbreviations_english.csv.gz')
WEB_GENRES = os.path.join(MY_DIR, 'web', 'key_genre_english.csv.gz')



__author__ = "Le Tuan Anh"
__email__ = "tuananh.ke@gmail.com"
__copyright__ = "Copyright 2018, freebible"
__credits__ = []
__license__ = "MIT License"
__description__ = "A bible processing toolkit with free bible data for Python"
__url__ = "https://github.com/neocl/freebible"
__maintainer__ = "Le Tuan Anh"
__version_major__ = "0.1"
__version__ = "{}a7".format(__version_major__)
__version_long__ = "{} - Alpha".format(__version_major__)
__status__ = "Prototype"



# -*- coding: utf-8 -*-

'''
Common bible processing functions
Latest version can be found at https://github.com/neocl/freebible

:copyright: (c) 2018, Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

import logging

from chirptext import TextReport

from freebible.data import KOUGO_PATH
from freebible.parsers.kougo import parse_kougo
from freebible.data import WEB_VERSES, WEB_BOOKS, WEB_ABBRS
from freebible.parsers.web import parse_web, read_abbr


# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

def getLogger():
    return logging.getLogger(__name__)


# -------------------------------------------------------------------------------
# Functions
# -------------------------------------------------------------------------------

def read_kougo():
    ''' Read Japanese Colloquial Bible (口語訳) '''
    return parse_kougo(KOUGO_PATH)


def read_web():
    ''' Read World English Bible '''
    return parse_web(WEB_VERSES, WEB_BOOKS, WEB_ABBRS)


def bookmap():
    ''' return World English Bible standard books mapping information (i.e. book names) '''
    return read_abbr(WEB_ABBRS)


class Bibles(object):

    def __init__(self):
        self.__kougo = None
        self.__web = None

    @property
    def kougo(self):
        ''' Singleton Kougo '''
        if not self.__kougo:
            self.__kougo = read_kougo()
        return self.__kougo

    @property
    def web(self):
        ''' Singleton WEB '''
        if not self.__web:
            self.__web = read_web()
        return self.__web

    def printlines(self, lines, output=None):
        if output:
            for line in lines:
                output.write(line)
                output.write('\n')

    def quote(self, book_key, cid=None, vid=None, output=None):
        quotes = [b.quote(book_key, cid, vid) for b in (self.kougo, self.web)]
        self.printlines(quotes, output=output)
        return quotes

    def print(self, book_key, cid=None, vid=None):
        ''' Quote bibles to standard output '''
        return self.quote(book_key, cid=cid, vid=vid, output=TextReport())


bibles = Bibles()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Test script for bible parsers

Latest version can be found at https://github.com/neocl/freebible

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2018, Le Tuan Anh <tuananh.ke@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

########################################################################

import os
import unittest
import logging

import freebible
from freebible.parsers import web
from freebible.parsers.web import read_keys, read_verses, read_abbr, read_genres

# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

TEST_DIR = os.path.dirname(os.path.realpath(__file__))


def getLogger():
    return logging.getLogger(__name__)

# -------------------------------------------------------------------------------
# Tests
# -------------------------------------------------------------------------------


class TestKougoParser(unittest.TestCase):

    def test_parse_kougo(self):
        bible = freebible.read_kougo()
        cc = sum(len(b) for b in bible)
        cv = sum(b.verse_count() for b in bible)
        self.assertEqual(len(bible), 66)
        self.assertEqual(cc, 1189)
        self.assertEqual(cv, 31105)
        self.assertEqual(len(bible.keys()), 66)


class TestWEBParser(unittest.TestCase):

    def test_read_web_raw(self):
        books = list(read_keys(freebible.data.WEB_BOOKS))
        self.assertEqual(len(books), 66)
        self.assertEqual(books[0]['title'], 'Genesis')
        # test read genres
        genres = read_genres(freebible.data.WEB_GENRES)
        self.assertEqual(genres['1'], 'Law')
        # test read abbrs
        abbrs = read_abbr(freebible.data.WEB_ABBRS)
        self.assertEqual(abbrs.bid2name('1'), 'Gen')
        verses = list(read_verses(freebible.data.WEB_VERSES))
        self.assertEqual(len(verses), 31102)
        self.assertTrue(verses[0]['text'].startswith('In the beginning'))

    def test_read_web(self):
        bible = freebible.read_web()
        cc = sum(len(b) for b in bible)
        cv = sum(b.verse_count() for b in bible)
        self.assertEqual(len(bible), 66)
        self.assertEqual(cc, 1189)
        self.assertEqual(cv, 31102)
        self.assertEqual(len(bible.keys()), 66)


class TestCrossCheck(unittest.TestCase):

    def test_kougo_web_keys(self):
        kougo = freebible.read_kougo()
        web = freebible.read_web()
        abbrs = read_abbr(freebible.data.WEB_ABBRS)
        for b in kougo:
            std_abbr = abbrs.standardize(b.short_name)
            self.assertTrue(std_abbr)
            if not std_abbr:
                getLogger().warning("{} - {} not found".format(b.title_eng, b.short_name))
            self.assertIn(std_abbr, web)
        for b in web:
            self.assertIn(b.short_name, kougo)

    def test_verses(self):
        kougo = freebible.read_kougo()
        web = freebible.read_web()
        abbrs = read_abbr(freebible.data.WEB_ABBRS)
        for b in web:
            for c in b:
                for v in c:
                    try:
                        t = kougo[b.short_name][c.ID][v.ID]
                        assert t
                    except:
                        getLogger().warning("Missing verse in kougo {} {} {}".format(b.short_name, c.ID, v.ID))
        for b in kougo:
            for c in b:
                for v in c:
                    try:
                        t = web[abbrs.standardize(b.short_name)][c.ID][v.ID]
                        assert t
                    except:
                        getLogger().warning("Missing verse in WEB {} {} {}".format(b.short_name, c.ID, v.ID))


class TestWEBNoteExtraction(unittest.TestCase):
    TEST_VERSE = "He called the name of the place Massah,{Massah means testing.} and Meribah,{Meribah means quarreling.} because the children of Israel quarreled, and because they tested Yahweh, saying, \"Is Yahweh among us, or not?\""

    def test_extract_note(self):
        notes = web.find_notes(self.TEST_VERSE)
        expected = [('Massah,', 'Massah means testing.'), ('Meribah,', 'Meribah means quarreling.')]
        self.assertEqual(notes, expected)

    def test_remove_notes(self):
        expected = "He called the name of the place Massah, and Meribah, because the children of Israel quarreled, and because they tested Yahweh, saying, \"Is Yahweh among us, or not?\""
        actual = web.strip_notes(self.TEST_VERSE)
        getLogger().debug("After: {}".format(actual))
        self.assertEqual(actual, expected)

    def test_verse_text(self):
        print("Test WEB verse without note")
        actual = freebible.bibles.web['Gen'][1][1].text
        expected = 'In the beginning God created the heavens and the earth.'
        self.assertEqual(actual, expected)


# -------------------------------------------------------------------------------
# Main
# -------------------------------------------------------------------------------

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
World English Bible parser
Latest version can be found at https://github.com/neocl/freebible

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2018, Le Tuan Anh <tuananh.ke@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

########################################################################

import logging
from collections import namedtuple
import re

from freebible.model import Collection, Book, Chapter, Verse, BookMap
from chirptext.io import read_csv, read_csv_iter, is_file


# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

def getLogger():
    return logging.getLogger(__name__)


# -------------------------------------------------------------------------------
# Functions
# -------------------------------------------------------------------------------

NOTE_WITH_WORD_REGEX = re.compile("\s(?P<word>\S*?){(?P<comment>.*?)}")
NOTE_ONLY_REGEX = re.compile("{(?P<comment>.*?)}")
Comment = namedtuple("Comment", ["word", "comment"])


def find_notes(verse_text):
    comments = []
    res = NOTE_WITH_WORD_REGEX.findall(verse_text)
    for m in res:
        cmt = Comment(*m)
        comments.append(cmt)
    return comments


def strip_notes(verse_text):
    return NOTE_ONLY_REGEX.sub('', verse_text)


def read_verses(path):
    ''' Read all WEB's raw verses from CSV file '''
    verses_iter = read_csv_iter(path, fieldnames=['ID', 'bookID', 'chapID', 'verseID', 'text'])
    next(verses_iter)
    return verses_iter


def read_keys(path):
    ''' Read books information from CSV file '''
    keys_iter = read_csv_iter(path, fieldnames=('ID', 'title', 'col', 'genreID'))
    next(keys_iter)
    return keys_iter


def parse_web(path, book_path, abbr_path=None):
    ''' Parse WEB (World English Bible) from raw text format '''
    if not is_file(path) or not is_file(book_path):
        raise Exception("WEB data could not be found (provided path: {})".format(path))
    verses = read_verses(path)
    keys = read_keys(book_path)
    abbrs = read_abbr(abbr_path) if abbr_path else None
    return parse_web_raw(verses, keys, abbrs=abbrs)


def read_abbr(path):
    ''' return a BookMap object (for converting between bookID and short_name) '''
    abbr_data = read_csv(path, fieldnames=('ID', 'text', 'bookID', 'is_standard'))[1:]
    return BookMap(abbr_data)


def read_genres(path):
    rows = read_csv(path, fieldnames=('ID', 'genre'))[1:]
    return {row['ID']: row['genre'] for row in rows}


def parse_web_raw(verses, books, abbrs=None):
    bible = Collection()
    # add books
    for b in books:
        short_name = abbrs.bid2name(b['ID'])
        book_obj = Book(ID=b['ID'], title=b['title'], title_eng=b['title'], short_name=short_name)
        bible.add_book(book_obj)
    # add verses
    for idx, v in enumerate(verses):
        book = bible.bookid_map[v['bookID']]
        cid = v['chapID']
        if cid not in book:
            # create a new chapter
            chapter = Chapter(ID=cid)
            book.add_chapter(chapter)
        else:
            chapter = book[cid]
        # add verse to chapter
        verse_text = strip_notes(v['text'])
        verse_obj = Verse(ID=v['verseID'], text=verse_text, chapID=cid, book_key=book.short_name)
        chapter.add_verse(verse_obj)
    return bible



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
A bible processing toolkit with free bible data for Python
Latest version can be found at https://github.com/neocl/freebible

:copyright: (c) 2018, Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

import os

from chirptext import TextReport
from chirptext.cli import CLIApp, setup_logging

from freebible.model import Collection
from freebible.data import KOUGO_PATH
from freebible.parsers.kougo import parse_kougo

# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

setup_logging('logging.json', 'logs')

MY_DIR = os.path.dirname(__file__)
DATA_FOLDER = os.path.join(MY_DIR, 'data')


# -------------------------------------------------------------------------------
# Application logic
# -------------------------------------------------------------------------------


def parse_bible(cli, args):
    ''' Parse raw bible data into freebible format '''
    rp = TextReport()
    if args.input:
        rp.print("Parsing kougo")
        kougo = parse_kougo(args.input)
        kougo.summarise()
    if args.output:
        rp.print("Exporting kougo to {}".format(args.output))
        with TextReport(args.output) as outfile:
            kougo.export(outfile, args.format)


def prompt():
    return input("Query (? or help): ")


def process(bible, query):
    try:
        if query in ('help', '?'):
            print("Available commands:")
            print("   + help: Display this help")
            print("   + books : List available books")
            print("   + quit or exit: Quit bible reader")
        elif query == 'books':
            print(' '.join(b.short_name for b in bible))
        elif query.lower() in ("quit", "exit"):
            print("Good bye. Have a nice day.")
            return False
        else:
            bn, cid, vid = query.split()
            print(bible[bn][cid][vid])
    except:
        print("Unknown command")
    return True


def read_bible(cli, args):
    if args.input and os.path.isfile(args.input):
        bible = Collection.read_json_file(args.input)
    else:
        # try to load from raw file
        bible = parse_kougo(KOUGO_PATH)
    bible.summarise()
    while True:
        query = prompt()
        if not process(bible, query):
            exit()


# -------------------------------------------------------------------------------
# Main method
# -------------------------------------------------------------------------------

def main():
    ''' freebible tools '''

    app = CLIApp(desc='Free Bible Toolkit for Python', logger=__name__)
    # add tasks
    task = app.add_task('parse', func=parse_bible)
    task.add_argument('-i', '--input', help='Input file', default=KOUGO_PATH)
    task.add_argument('-o', '--output', help='Output file')
    task.add_argument('-f', '--format', help='Output format (json, yaml, txt)', default='json', choices=['json', 'yaml', 'txt'])
    # bible read
    task = app.add_task('read', func=read_bible)
    task.add_argument('-i', '--input', help='Input file (JSON format)')
    # run app
    app.run()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

'''
Python Free Bible Toolkits
Latest version can be found at https://github.com/neocl/freebible

:copyright: (c) 2018, Le Tuan Anh <tuananh.ke@gmail.com>
:license: MIT, see LICENSE for more details.
'''

from .__version__ import __author__, __email__, __copyright__, __maintainer__
from .__version__ import __credits__, __license__, __description__, __url__
from .__version__ import __version_major__, __version_long__, __version__, __status__

from freebible.helpers import read_kougo, read_web, bibles
from freebible.model import Collection
__all__ = ['read_kougo', 'read_web', 'bibles', 'Collection',
           "__version__", "__author__", "__description__", "__copyright__"]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Test script for heplpers module

Latest version can be found at https://github.com/neocl/freebible

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2018, Le Tuan Anh <tuananh.ke@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

########################################################################

import os
import unittest
import logging

from chirptext import TextReport

import freebible

# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

TEST_DIR = os.path.dirname(os.path.realpath(__file__))


def getLogger():
    return logging.getLogger(__name__)


# -------------------------------------------------------------------------------
# Tests
# -------------------------------------------------------------------------------

class TestHelpers(unittest.TestCase):

    def test_bibles(self):
        self.assertEqual(len(freebible.bibles.kougo), 66)
        self.assertEqual(len(freebible.bibles.web), 66)

    def test_quoting(self):
        output = TextReport()
        books = freebible.bibles.quote('John', output=output)
        self.assertTrue(books)
        for b in books:
            self.assertIsInstance(b, freebible.model.Book)
        chapters = freebible.bibles.quote('John', 1, output=output)
        self.assertTrue(chapters)
        for c in chapters:
            self.assertIsInstance(c, freebible.model.Chapter)
        verses = freebible.bibles.quote('John', 1, 1, output=output)
        self.assertTrue(verses)
        for v in verses:
            self.assertIsInstance(v, freebible.model.Verse)

    def test_printing(self):
        freebible.bibles.print('John')
        freebible.bibles.print('John', 1)
        freebible.bibles.print('John', 1, 1)


# -------------------------------------------------------------------------------
# Main
# -------------------------------------------------------------------------------

if __name__ == "__main__":
    unittest.main()
